# Linux specific rules

# Rule to build an executable from a set of .o files
ifdef EX_TARGET
$(EX_TARGET): $(EX_OBJS_$@)
	$(CC) $(CFLAGS) $(EX_CFLAGS_$@) -o $@ $(EX_OBJS_$@) $(LDFLAGS) $(EX_LDFLAGS_$@)
endif

# Rule to build a shared library from a set of .o files
# Note that only the major is used when setting the soname, this
# way any minor changes to the libraries can be made without forcing a
# relink.. when installed the entire major.minor.path is appended to the
# file
ifdef SO_TARGET
$(SO_TARGET):
	$(CC) -nostdlib -Wl,-shared -Wl,-soname,$@.$(SO_MAJOR_$@) $(SO_CFLAGS_$@) -o $@ $(SO_OBJS_$@) $(LDFLAGS) $(SO_LDFLAGS_$@)
endif

# Plugin target, this is for Warsaw's plugins, they
# don't require any major, minor or patch level and they get installed
# into a special directory.
ifdef PLUG_TARGET
$(PLUG_TARGET):
	$(CC) -nostdlib -Wl,-shared -Wl,-soname,$@ $(PLUG_CFLAGS_$@) -o $@ $(PLUG_OBJS_$@) $(LDFLAGS) $(PLUG_LDFLAGS_$@)
endif

# Rule to build an archive .a file from a set of .o files
ifdef AR_TARGET
$(AR_TARGET):
	$(AR) $@ $(AR_OBJS_$@)
	$(RANLIB) $@
endif

# Rule to install Berlin
# We want to put Berlin in /usr because it's as important as X
# However, UNIX standard directories aren't really good for packages
# since the UNIX paradigm is small programs working together. Sure
# wish we had /opt :)

# Anyway, this installs in the UNIX way, however we may want to change
# this one day to install the subprograms in their own special
# directories.
install: all $(OS_INSTALL_DIRS)
ifdef SO_TARGET
ifneq ($(SO_INSTALL_$(SO_TARGET)),no)
	$(foreach file, $(SO_TARGET), $(INSTALL) -m755 $(file) $(LIB_DIR)/$(file).$(SO_MAJOR_$(file)).$(SO_MINOR_$(file)).$(SO_PATCH_$(file));)
endif
endif
ifdef PLUG_TARGET
ifneq ($(PLUG_INSTALL_$(PLUG_TARGET)),no)
	$(foreach file, $(PLUG_TARGET), $(INSTALL) -m755 $(file) $(PLUG_DIR)/$(PLUG_SUBDIR_$(file));)
endif
endif
ifdef EX_TARGET
ifneq ($(EX_INSTALL_$(EX_TARGET)),no)
	$(foreach file, $(EX_TARGET), $(INSTALL) -m755 $(file) $(BIN_DIR);)
endif
endif
ifdef CO_TARGET
ifneq ($(CO_INSTALL_$(CO_TARGET)),no)
	$(foreach file, $(CO_TARGET), $(INSTALL) -m644 $(file) $(CONF_DIR);)
endif
endif
ifdef TX_TARGET
ifneq ($(TX_INSTALL_$(TX_TARGET)),no)
	$(foreach file, $(TX_TARGET), $(INSTALL) -m644 $(file) $(DOC_DIR);)
endif
endif
	@$(MakeSubdirs)

# This creates $(prefix) + supporting directories
$(OS_INSTALL_DIRS):
	$(MKDIR) $@
