# This file determines the computers architecture and platform and
# includes the most appropriate .mk file from the config/ directory. There
# isn't too much OS specific stuff in here (other than arch determination
# code)

OS_ARCH		:= $(subst /,_,$(shell uname -s))
OS_RELEASE	:= $(shell uname -r)
OS_TEST		:= $(shell uname -m)

# Tweak the default OS_ARCH and OS_RELEASE macros as needed.

ifeq ($(OS_ARCH),AIX)
OS_RELEASE	:= $(shell uname -v).$(shell uname -r)
endif
ifeq ($(OS_ARCH),BSD_386)
OS_ARCH		:= BSD_OS
endif
ifeq ($(OS_ARCH),IRIX64)
OS_ARCH		:= IRIX
endif
ifeq ($(OS_ARCH),UNIX_SV)
ifneq ($(findstring NCR,$(shell grep NCR /etc/bcheckrc | head -1 )),)
OS_ARCH		:= NCR
else
OS_ARCH		:= UNIXWARE
OS_RELEASE	:= $(shell uname -v)
endif
endif
ifeq ($(OS_ARCH),ncr)
OS_ARCH		:= NCR
endif
# This is the only way to correctly determine the actual OS version on NCR boxes.
ifeq ($(OS_ARCH),NCR)
OS_RELEASE	:= $(shell awk '{print $$3}' /etc/.relid | sed 's/^\([0-9]\)\(.\)\(..\)\(.*\)$$/\2.\3/')
endif
ifeq ($(OS_ARCH),UNIX_System_V)
OS_ARCH		:= NEC
endif
ifeq ($(OS_ARCH),SCO_SV)
OS_ARCH		:= SCOOS
OS_RELEASE	:= 5.0
endif
ifeq ($(OS_ARCH),SINIX-N)
OS_ARCH		:= SINIX
endif
# SINIX changes name to ReliantUNIX with 5.43
ifeq ($(OS_ARCH),ReliantUNIX-N)
OS_ARCH		:= SINIX
endif
ifeq ($(OS_ARCH),UnixWare)
OS_ARCH		:= UNIXWARE
endif

ifneq (,$(filter FreeBSD HP-UX IRIX Linux OSF1 SunOS,$(OS_ARCH)))
OS_VERS		:= $(suffix $(OS_RELEASE))
OS_RELEASE	:= $(basename $(OS_RELEASE))
OS_VERSION	:= $(shell echo $(OS_VERS) | sed 's/-.*//')
endif

OS_CONFIG       := $(OS_ARCH)$(OS_RELEASE)

# These are the default command names. They can and will be overrided by
# <arch>.mk files in the config/ directory.

CC            = g++
CXX           = g++
ECHO          = echo
RM            = rm -f
CDEPEND       = gcc -M
CXXDEPEND     = g++ -M
IDL           = omniidl2
INSTALL       = ginstall -c
AR            = ar cq
RANLIB        = ranlib
MKDIR         = mkdir -p

# Now include myconfig.mk if it exists
-include $(TOPDIR)/config/myconfig.mk

# Where we are going to install
ifndef prefix
prefix           = /usr/Berlin
endif

# Now include the <arch>/config.mk file
include $(TOPDIR)/config/$(OS_ARCH)/config.mk
