# This file defines rules used by all Makefiles to build this
# source. These are generic routines *ONLY*. Everything remotely OS
# specific is in config/<os>/rules.mk

.PHONY: all depend config skeletons

include $(TOPDIR)/config/config.mk

ifdef SUBDIRS
define MakeSubdirs
	(set -e; \
	for d in $(SUBDIRS); do \
		$(MAKE) -C $$d $@; \
	done;)
endef
endif

# this pass has to happen before regular compilation
ifdef SUBDIRS
define MakeSubSkeletons
	(set -e; \
	for d in $(SUBDIRS); do \
		$(MAKE) -C $$d skeletons; \
	done;)
endef
endif


CFLAGS := $(OS_CPPFLAGS) $(DIR_CPPFLAGS) $(OS_INCLUDES) $(DIR_INCLUDES) $(OS_CFLAGS) $(DIR_CFLAGS)
CXXFLAGS := $(OS_CPPFLAGS) $(DIR_CPPFLAGS) $(OS_INCLUDES) $(DIR_INCLUDES) $(OS_CXXFLAGS) $(DIR_CXXFLAGS)
IDLFLAGS := $(OS_IDLFLAGS) $(DIR_IDLFLAGS)
LDFLAGS := $(OS_LDFLAGS) $(DIR_LDFLAGS)

ifdef IDLSRCS
CXXSRCS := $(CXXSRCS) $(IDLSRCS:.idl=SK.cc)
endif


# Rule to build .o files from .c files
%.o: %.c
	$(CC) $(CFLAGS) -c $<

# Rule to build .o files from .cc files
%.o: %.cc
	$(CXX) $(CXXFLAGS) -c $<

# Rule to build C++ skeleton SK.cc files & headers from CORBA .idl files
%.hh %SK.cc: %.idl
	$(IDL) $(IDLFLAGS) $<

# Rules executed when entering a subdirectory... these are done
# in order. 


all:: skeletons
all:: all_targets

all_targets:: $(CSRCS:.c=.o) $(CXXSRCS:.cc=.o) $(SO_TARGET) $(AR_TARGET) $(EX_TARGET) $(PLUG_TARGET)
	@$(MakeSubdirs)

skeletons:: $(IDLSRCS:.idl=SK.cc) $(IDLSRCS:.idl=.hh)
	@$(MakeSubdirs)

# Simple dependancy generation.
depend: Makefile skeletons $(CSRCS) $(CXXSRCS)
	@$(MakeSubdirs)
	@$(RM) .depend
ifdef CSRCS
	$(CDEPEND) $(CFLAGS) $(CSRCS) >> .depend
endif
ifdef CXXSRCS
	$(CXXDEPEND) $(CXXFLAGS) $(CXXSRCS) >> .depend
endif

define CleanDir
	(remove="$(wildcard $(EX_TARGET) $(SO_TARGET) $(AR_TARGET) $(PLUG_TARGET) \
                 $(IDLSRCS:.idl=SK.cc) $(IDLSRCS:.idl=.hh) .depend *.o)"; \
	if [ "$$remove" != "" ]; then \
		$(ECHO) Removing: $$remove; \
		$(RM) $$remove; \
	fi;)
endef

clean:
	@$(MakeSubdirs)
	@$(CleanDir)

config:
	$(TOPDIR)/config/configure

ifeq (.depend,$(wildcard .depend))
include .depend
endif

include $(TOPDIR)/config/$(OS_ARCH)/rules.mk
