//
// $id:$
//
// This source file is a part of the Berlin Project.
// Copyright (C) 1998 The Berlin Consortium 
// http://www.berlin-consortium.org
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// Author: ANOQ of the Sun
// EMail: anoq@berlin-consortium.org or anoq@vip.cybercity.dk

#include "domAttrib_impl.hh"
#include "domString.hh"

domAttrib_impl::domAttrib_impl(const wstring & name, const wstring & value)
{
    _attrName = domStringDup(&name);
    _attrValue = domStringDup(&value);
    _specified = 0;
}

domAttrib_impl::~domAttrib_impl()
{
    domStringFree(_attrName);
    domStringFree(_attrValue);
}

CORBA::UShort domAttrib_impl::nodeType(void)
{
    return Node::ATTRIBUTE_NODE;
}

Node_ptr domAttrib_impl::cloneNode(CORBA::Boolean deep)
{
    Node_ptr newClone =
        new domAttrib_impl((const wstring &) *_attrName,
                           (const wstring &) *_attrValue);
    if(deep)
      deepCloneChildren(newClone);
    return newClone;
};

wstring * domAttrib_impl::nodeName( void)
{
    return domStringDup(_attrName);
};

wstring * domAttrib_impl::nodeValue( void)
{
    return domStringDup(_attrValue);
};

void domAttrib_impl::nodeValue(const wstring & _value)
{
    wstring * tmp_value = _attrValue;
    _attrValue = domStringDup(&_value);
    domStringFree(tmp_value);
};

CORBA::Boolean domAttrib_impl::mayInsertNode(CORBA::UShort nodeType)
{
    if(nodeType == Node::TEXT_NODE ||
       nodeType == Node::ENTITY_REFERENCE_NODE)
        return 1;
    else
        return 0;
}

wstring * domAttrib_impl::name( void)
{
    return nodeName();
};

CORBA :: Boolean domAttrib_impl::specified( void)
{
    return _specified;
};

wstring * domAttrib_impl::value( void)
{
    return nodeValue();
};

void domAttrib_impl::value(const wstring & _value)
{
    nodeValue(_value);
};

