//
// $id:$
//
// This source file is a part of the Berlin Project.
// Copyright (C) 1998 The Berlin Consortium 
// http://www.berlin-consortium.org
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// Author: ANOQ of the Sun
// EMail: anoq@berlin-consortium.org or anoq@vip.cybercity.dk

#include "domDocument_impl.hh"

domDocument_impl::~domDocument_impl(void)
{
}

CORBA::UShort domDocument_impl::nodeType( void)
{
    return Node::DOCUMENT_NODE;
};

Node_ptr domDocument_impl::cloneNode(CORBA::Boolean deep)
{
    Document_ptr newClone = new domDocument_impl();
    if(deep)
        deepCloneChildren(newClone);
    return newClone;
}

wstring * domDocument_impl::nodeName( void)
{
    return cStringToDOMString("#document");
};

wstring * domDocument_impl::nodeValue( void)
{
    return (wstring *)NULL;//Must return NULL according to DOM spec.
};

void domDocument_impl::nodeValue(const wstring & _value)
{
    //Does not do anything in Document according to DOM spec.
};

CORBA::Boolean domDocument_impl::mayInsertNode(CORBA::UShort nodeType)
{
    if(nodeType == Node::PROCESSING_INSTRUCTION_NODE ||
       nodeType == Node::COMMENT_NODE ||
       nodeType == Node::DOCUMENT_TYPE_NODE)
        return 1;
    else if(nodeType == Node::ELEMENT_NODE)
        return (documentElement() == (Element_ptr)NULL);//Only 1 Element node allowed
    else
        return 0;
}

DocumentType_ptr domDocument_impl::doctype( void)
{
    return (DocumentType_ptr)0;//FIXME: Not implemented
};

DOMImplementation_ptr domDocument_impl::implementation( void)
{
    return (DOMImplementation_ptr)0;//FIXME: Not implemented
    //Should maintain 1 static instance of DOMImplementation
};

Element_ptr domDocument_impl::documentElement( void)
{
    Node_ptr curr = firstChild();

    while(curr)
    {
        if(curr->nodeType() == Node::ELEMENT_NODE)
            return (Element_ptr)(void *)curr;
        curr = curr->nextSibling();
    }
    return (Element_ptr)NULL;//No Element node found...
};

Element_ptr domDocument_impl::createElement(const wstring & tagName)
{
    return new domElement_impl(tagName);
};

DocumentFragment_ptr domDocument_impl::createDocumentFragment( void)
{
    return new domDocumentFragment_impl();
};

Text_ptr domDocument_impl::createTextNode(const wstring & data)
{
    Text_ptr newTextNode = new domText_impl();
    newTextNode->nodeValue(data);
    return newTextNode;
};

Comment_ptr domDocument_impl::createComment(const wstring & data)
{
    Comment_ptr newCommentNode = new domComment_impl();
    newCommentNode->nodeValue(data);
    return newCommentNode;
};

CDATASection_ptr domDocument_impl::createCDATASection(const wstring & data)
{
    CDATASection_ptr newCDATANode = new domCDATASection_impl();
    newCDATANode->nodeValue(data);
    return newCDATANode;
};

ProcessingInstruction_ptr domDocument_impl::createProcessingInstruction(const wstring & name ,const wstring & data)
{
    ProcessingInstruction_ptr newPINode = new domProcessingInstruction_impl(name);
    newPINode->nodeValue(data);
    return newPINode;
};

Attrib_ptr domDocument_impl::createAttribute(const wstring & name)
{
    wstring *tmpValue = cStringToDOMString("");
    Attrib_ptr newAttrib = new domAttrib_impl(name, *tmpValue);
    delete tmpValue;//FIXME: Make better constructor for Attrib

    return newAttrib;
};

EntityReference_ptr domDocument_impl::createEntityReference(const wstring & name)
{
    return new domEntityReference_impl();//FIXME: Not complete yet!
}

NodeList_ptr domDocument_impl::getElementsByTagName(const wstring & tagname)
{
    return (NodeList_ptr)0;//FIXME: Not implemented yet
};

