//
// $id:$
//
// This source file is a part of the Berlin Project.
// Copyright (C) 1998 The Berlin Consortium 
// http://www.berlin-consortium.org
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// Author: ANOQ of the Sun
// EMail: anoq@berlin-consortium.org or anoq@vip.cybercity.dk

#include "domElement_impl.hh"
#include "domNamedNodeMap_impl.hh"
#include <Document.hh>
#include <stdio.h>

domElement_impl::domElement_impl(const wstring & tagName)
{
    _tagName = domStringDup(&tagName);
    _attribList = new domNamedNodeMap_impl();
}

domElement_impl::~domElement_impl()
{
    domStringFree(_tagName);
    //FIXME: Free all attributes?
    //if(_attribList)
    //    delete _attribList;
}

CORBA::UShort domElement_impl::nodeType( void)
{
    return Node::ELEMENT_NODE;
};

Node_ptr domElement_impl::cloneNode(CORBA::Boolean deep)
{
    Element_ptr newClone = new domElement_impl(*_tagName);

    //Copy attributes... FIXME: An efficient Element constructor should be
    //created here instead, to achieve better performance!!!
    for(CORBA::ULong cnt = 0; cnt < _attribList->length();cnt++)
        newClone->setAttributeNode((Attrib_ptr)(void *)_attribList->item(cnt));
    //FIXME: Better typecast preferable!
    
    if(deep)
        deepCloneChildren(newClone);
    return newClone;
}

wstring * domElement_impl::nodeName(void)
{
    return domStringDup(_tagName);
}

wstring * domElement_impl::nodeValue(void)
{
    return (wstring *)NULL;//This is correct according to DOM spec.
}

void domElement_impl::nodeValue(const wstring & _value)
{
    //Does not do anything in Element according to DOM spec.
}

CORBA::Boolean domElement_impl::mayInsertNode(CORBA::UShort nodeType)
{
    if(nodeType == Node::ELEMENT_NODE ||
       nodeType == Node::PROCESSING_INSTRUCTION_NODE ||
       nodeType == Node::COMMENT_NODE ||
       nodeType == Node::TEXT_NODE ||
       nodeType == Node::CDATA_SECTION_NODE ||
       nodeType == Node::ENTITY_REFERENCE_NODE)
        return 1;
    else
        return 0;
}

wstring * domElement_impl::tagName( void)
{
    return nodeName();
};

NamedNodeMap_ptr domElement_impl::attributes(void)
{
    return _attribList;//FIXME: Must be live (and readonly?)
}

NamedNodeMap_ptr domElement_impl::getAttributes( void)
{
    return attributes();
};

wstring * domElement_impl::getAttribute(const wstring & name)
{
    Attrib_ptr attr = getAttributeNode(name);
    if(attr)
        return attr->nodeValue();
    else
        return (wstring *)NULL;
};

void domElement_impl::setAttribute(const wstring & name ,const wstring & value)
{
    Document_ptr masterDoc = (Document_ptr)(void *)ownerDocument();
    if(!masterDoc)
        return;
    
    Attrib_ptr attr = masterDoc->createAttribute(name);
    
    if(attr)
    {
        attr->nodeValue(value);
        setAttributeNode(attr);
    }
    else
        puts("Cannot create attribute in domElement_impl::setAttribute");
    //FIXME: Exception?
};

void domElement_impl::removeAttribute(const wstring & name)
{
    if(!_attribList)
        throw DOMException(NOT_FOUND_ERR);//FIXME: Is this correct?

    _attribList->removeNamedItem(name);//FIXME: Free returned attribute?
};

Attrib_ptr domElement_impl::getAttributeNode(const wstring & name)
{
    if(!_attribList)
        throw DOMException(NOT_FOUND_ERR);//FIXME: Is this correct?
    
    return (Attrib_ptr)(void *)_attribList->getNamedItem(name);
};

Attrib_ptr domElement_impl::setAttributeNode(const Attrib_ptr newAttr)
{
    _attribList->setNamedItem(newAttr);
    return newAttr;//FIXME: Is this correct?
};

Attrib_ptr domElement_impl::removeAttributeNode(const Attrib_ptr oldAttr)
{
    if(!oldAttr)
        throw DOMException(NOT_FOUND_ERR);//FIXME: Is this correct?

    //FIXME: Is this correct?
    wstring *name = oldAttr->nodeName();
    try
    {
        removeAttribute(*name);
    }
    catch(...)
    {
        delete name;
        throw;
    }
    delete name;

    return oldAttr;//FIXME: Is this correct?
};

NodeList_ptr domElement_impl::getElementsByTagName(const wstring & name)
{
    return (NodeList_ptr)NULL;//FIXME: Not implemented yet
};

void domElement_impl::normalize( void)
{
    //FIXME: Not implemented yet...
};

