//
// $id:$
//
// This source file is a part of the Berlin Project.
// Copyright (C) 1998 The Berlin Consortium 
// http://www.berlin-consortium.org
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// Author: ANOQ of the Sun
// EMail: anoq@berlin-consortium.org or anoq@vip.cybercity.dk

#include "domNamedNodeMap_impl.hh"
#include <DOMException.hh>
#include "domString.hh"
#include <malloc.h>

#define _NODEMAP_EXPAND 32

//FIXME: This must be a live NamedNodeMap!

domNamedNodeMap_impl::~domNamedNodeMap_impl()
{
    if(_nodeArray)
        free(_nodeArray);
}

Node_ptr domNamedNodeMap_impl::getNamedItem(const wstring & name)
{
    bool same;
    
    if(!_nodeArray)
        return (Node_ptr)NULL;

    for(CORBA::ULong cnt = 0 ; cnt < _arrLength ; cnt++)
    {
        wstring *currName = _nodeArray[cnt]->nodeName();

        same = domStringEqual(currName, &name);

        domStringFree(currName);

        if(same)
            return _nodeArray[cnt];
    }
    
    return (Node_ptr)NULL;
};

void domNamedNodeMap_impl::setNamedItem(Node_ptr arg)
{
    //Exceptions
    //  DOMException
    //    WRONG_DOCUMENT_ERR: Raised if arg was created from a different document than the one that created the NamedNodeMap.
    //    NO_MODIFICATION_ALLOWED_ERR: Raised if this NamedNodeMap is readonly.
    //    INUSE_ATTRIBUTE_ERR: Raised if arg is an Attribute that is already attribute of another Element object. The DOM user must explicitly clone
    //      Attribute nodes to re-use them in other elements.
    
    //Check expand / alloc array
    if(_arrLength >= _arrSize)
    {
        CORBA::ULong newSize = _arrSize + _NODEMAP_EXPAND;
        Node_ptr *newArr = (Node_ptr *)malloc(newSize * sizeof(Node_ptr));
        if(!newArr)
            throw CORBA::NO_MEMORY();//FIXME: Must be defined in IDL!
        
        if(_nodeArray)
        {
            //Copy data from old array to new array
            for(CORBA::ULong cnt = 0; cnt < _arrSize; cnt++)
                newArr[cnt] = _nodeArray[cnt];
            
            free(_nodeArray);
        }
        _nodeArray = newArr;
        _arrSize = newSize;
    }

    //Replace or add attribute
    bool same = 0;
    Node_ptr oldNode = (Node_ptr)NULL;
    wstring *nodeName = arg->nodeName();
    
    for(CORBA::ULong cnt = 0 ; cnt < _arrLength && !same ; cnt++)
    {
        wstring *name = _nodeArray[cnt]->nodeName();

        same = domStringEqual(name, nodeName);

        domStringFree(name);

        if(same)
        {
            //FIXME: Is this correct? Perhaps the value should just be updated?
            oldNode = _nodeArray[cnt];
            _nodeArray[cnt] = arg;
        }
    }
    delete nodeName;

    if(!same)
    {
        //Add attribute
        _nodeArray[_arrLength] = arg;
        _arrLength++;
    }
    
    //return oldNode;
};

Node_ptr domNamedNodeMap_impl::removeNamedItem(const wstring & name)
{
    bool same;
    
    if( ! &name)
        throw DOMException(NOT_FOUND_ERR);

    for(CORBA::ULong cnt = 0 ; cnt < _arrLength ; cnt++)
    {
        wstring *currName = _nodeArray[cnt]->nodeName();

        same = domStringEqual(currName, &name);

        domStringFree(currName);

        if(same)
        {
            Node_ptr oldNode = _nodeArray[cnt];

            //Concat array elements
            for(CORBA::ULong cnt2 = cnt + 1; cnt2 < _arrLength ; cnt2++)
                _nodeArray[cnt2 - 1] = _nodeArray[cnt2];

            _arrLength--;
            
            return oldNode;
        }
    }
    
    throw DOMException(NOT_FOUND_ERR);
};

Node_ptr domNamedNodeMap_impl::item(CORBA::ULong index)
{
    if(index >= _arrLength)
        return (Node_ptr)NULL;

    return _nodeArray[index];
};

CORBA :: ULong domNamedNodeMap_impl::length( void)
{
    return _arrLength;
};

