//
// $id:$
//
// This source file is a part of the Berlin Project.
// Copyright (C) 1998 The Berlin Consortium 
// http://www.berlin-consortium.org
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// Author: ANOQ of the Sun
// EMail: anoq@berlin-consortium.org or anoq@vip.cybercity.dk

#include "domNode_impl.hh"
#include "domString.hh"
#include "domNodeList_impl.hh"

void domNode_impl::deepCloneChildren(Node_ptr clonedNode)
{
    ASS(clonedNode);
    Node_ptr curr = firstChild();

    while(curr)
    {
        clonedNode->appendChild(curr->cloneNode(true));
        curr = curr->nextSibling();
    }
}

Node_ptr domNode_impl::ownerDocument(void)
{
    return _ownerDocument;
}


Node_ptr domNode_impl::parentNode( void)
{
    return _parentNode;
};

NodeList_ptr domNode_impl::childNodes( void)
{
    return new domNodeList_impl(_firstChild);
};

Node_ptr domNode_impl::firstChild( void)
{
    return _firstChild;
};

Node_ptr domNode_impl::lastChild( void)
{
    //FIXME: Very inefficient!
    Node_ptr curr = _firstChild;
    Node_ptr tmpNew;

    while(curr)
    {
        tmpNew = curr->nextSibling();
        if(tmpNew)
            curr = tmpNew;
    }
    return curr;
};

Node_ptr domNode_impl::previousSibling( void)
{
    return _previousSibling;
};

Node_ptr domNode_impl::nextSibling( void)
{
    return _nextSibling;
};

NamedNodeMap_ptr domNode_impl::attributes( void)
{
    return (NamedNodeMap_ptr)NULL;
    //Only returns a NamedNodeMap in the Element
    //subclass according to DOM spec
};

CORBA::Boolean domNode_impl::mayInsertNode(CORBA::UShort nodeType)
{
    return 1;
}

Node_ptr domNode_impl::insertBefore(Node_ptr newChild ,Node_ptr refChild)
{
    Node_ptr curr = _firstChild;
    Node_ptr prevNode = (Node_ptr)NULL;

    if(!mayInsertNode(newChild->nodeType()))
        throw DOMException(HIERARCHY_REQUEST_ERR);
    
    if(!curr || !refChild)
        throw DOMException(NOT_FOUND_ERR);//FIXME: Is this correct?
    
    while(curr != refChild)
    {
        prevNode = curr;
        curr = curr->nextSibling();
        if(!curr)
            throw DOMException(NOT_FOUND_ERR);//FIXME: Is this correct?
    }
    if(prevNode != curr->previousSibling())
        puts("Internal inconsistency in domNode_impl::insertBefore!");
    newChild->setParentNode__(this);
    newChild->setPreviousSibling__(prevNode);
    newChild->setNextSibling__(curr);
    if(prevNode)
        prevNode->setNextSibling__(newChild);
    else
        _firstChild = newChild;//This is inserted as the first child
    curr->setPreviousSibling__(newChild);
    
    return newChild;
};

Node_ptr domNode_impl::replaceChild(Node_ptr newChild ,Node_ptr oldChild)
{
    Node_ptr curr = _firstChild;
    Node_ptr prevNode = (Node_ptr)NULL;

    if(!mayInsertNode(newChild->nodeType()))
        throw DOMException(HIERARCHY_REQUEST_ERR);
    
    if(!curr || !oldChild)
        throw DOMException(NOT_FOUND_ERR);//FIXME: Is this correct?
    
    while(curr != oldChild)
    {
        prevNode = curr;
        curr = curr->nextSibling();
        if(!curr)
            throw DOMException(NOT_FOUND_ERR);//FIXME: Is this correct?
    }
    Node_ptr nextNode = curr->nextSibling();
    if(prevNode != curr->previousSibling())
        puts("Internal inconsistency in domNode_impl::replaceChild!");
    newChild->setParentNode__(this);
    newChild->setPreviousSibling__(prevNode);
    newChild->setNextSibling__(nextNode);
    if(prevNode)
        prevNode->setNextSibling__(newChild);
    else
        _firstChild = newChild;//This is inserted as the first child
    if(nextNode)
        nextNode->setPreviousSibling__(newChild);
    oldChild->setPreviousSibling__((Node_ptr)NULL);
    oldChild->setNextSibling__((Node_ptr)NULL);
    oldChild->setParentNode__((Node_ptr)NULL);//FIXME: What happens to orphan nodes?
    
    return oldChild;
};

Node_ptr domNode_impl::removeChild(Node_ptr oldChild)
{
    Node_ptr curr = _firstChild;
    Node_ptr prevNode = (Node_ptr)NULL;

    if(!curr || !oldChild)
            throw DOMException(NOT_FOUND_ERR);//FIXME: Is this correct?

    while(curr != oldChild)
    {
        prevNode = curr;
        curr = curr->nextSibling();
        if(!curr)
            throw DOMException(NOT_FOUND_ERR);//FIXME: Is this correct?
    }
    if(prevNode != curr->previousSibling())
        puts("Internal inconsistency in domNode_impl::removeChild!");
    Node_ptr nextNode = curr->nextSibling();
    if(prevNode)
        prevNode->setNextSibling__(nextNode);
    else
        _firstChild = nextNode;
    if(nextNode)
        nextNode->setPreviousSibling__(prevNode);
    curr->setPreviousSibling__((Node_ptr)NULL);
    curr->setNextSibling__((Node_ptr)NULL);
    curr->setParentNode__((Node_ptr)NULL);
    
    return oldChild;
};

Node_ptr domNode_impl::appendChild(Node_ptr newChild)
{
    if(!mayInsertNode(newChild->nodeType()))
        throw DOMException(HIERARCHY_REQUEST_ERR);
    
    Node_ptr prevNode = lastChild();
    
    newChild->setParentNode__(this);
    newChild->setPreviousSibling__(prevNode);
    newChild->setNextSibling__((Node_ptr)NULL);
    if(prevNode)
        prevNode->setNextSibling__(newChild);
    else
        _firstChild = newChild;//This is inserted as the first child

    return newChild;
};

CORBA::Boolean domNode_impl::hasChildNodes( void)
{
    return (CORBA::Boolean)(_firstChild != (Node_ptr)NULL);
};

void domNode_impl::setParentNode__(Node_ptr newParent)
{
    _parentNode = newParent;
};

void domNode_impl::setPreviousSibling__(Node_ptr newPrevSib)
{
    _previousSibling = newPrevSib;
};

void domNode_impl::setNextSibling__(Node_ptr newNextSib)
{
    _nextSibling = newNextSib;
};

